class BaseParsingError(Exception):
    """Exception base para as exceções no HTML

    Attributes:
        mensagem -- Mensagem da exceção
    """
    def __init__(self, mensagem: str):
        self.message = mensagem
        super().__init__(self.message)


class TagNotFoundError(BaseParsingError):
    """Exceção levantada quando o HTML não possui a tag solicitada

    Attributes:
        tag -- Nome da tag não encontrada
    """

    def __init__(self, tag: str):
        self.tag = tag
        super().__init__(f'Tag {tag} não encontrada')


class EmptyTagValueError(BaseParsingError):
    """Excecção levantada quando a tag HTML não tem `value` configurada

    Attributes:
        tag -- Nome da tag sem `value`
    """

    def __init__(self, tag):
        self.tag = tag
        super().__init__(f"Tag {tag} não possui value")


class PatternNotFoundError(BaseParsingError):
    """Exceção levantada quando o pattern não foi encontrado
    na resposta do servidor, impedindo de prosseguir a consulta

    Attributes:
        nome -- nome do pattern procurado
        regex -- regex usado para buscar o pattern
    """

    def __init__(self, nome: str, regex: str):
        self.nome = nome
        self.regex = regex
        super().__init__(
            f"{nome} não foi encontrado no HTML (regex: {regex})"
        )


class NotCSVError(BaseParsingError):
    """Exceção levantada quando a consulta final
    não retornou um CSV.

    Isso pode acontecer quando os
    headers ou o payload da consulta estão incorretos.
    """

    def __init__(self):
        super().__init__("Consulta final não retornou um CSV")


class WebExceptionError(BaseParsingError):
    """Exceção levantada quando a consulta inicial
    retorna uma página de erro.

    Isso pode acontecer quando os
    headers ou o payload da consulta estão incorretos.
    """

    def __init__(self, mensagem: str = ""):
        if not mensagem:
            mensagem = "Primeira consulta retornou uma página de erro"
        super().__init__(mensagem)



__all__ = ["PayloadMicrohorario", "PayloadModo"]

from enum import Enum, auto


class PayloadModo(Enum):
    """Modo de utilização dos Payloads"""
    MICROHORARIO = auto()       # utiliza a configuração padrão
    HORARIO = auto()            # utiliza a configuração alternativa


class PayloadMicrohorario:
    _INTERMEDIARIO = {
        "ScriptManager1": "pnlConteudo|btnBuscar",
        "__EVENTTARGET": "",
        "__EVENTARGUMENT": "",
        "ddlNivel": "Sem Distinção",
        "txtCodigoDptDcp": "",
        "txtNomeDcp": "",
        "txtQtdCreditos": "",
        "txtNomeProfessor": "",
        "ddlBloqueio": "-1",
        "ddlDia": "Qualquer dia",
        "txtHoraInicio": "",
        "txtHoraFim": "",
        "ddlTurno": "-1",
        "ddlDeptoSolicitante": "-1",
        "hiddenInputToUpdateATBuffer_CommonToolkitScripts": "1",
        "__ASYNCPOST": "true",
        "btnBuscar": "Buscar",
    }

    _FINAL = {
        "ddlNivel": "Sem Distinção",
        "txtCodigoDptDcp": "",
        "txtNomeDcp": "",
        "txtQtdCreditos": "",
        "txtNomeProfessor": "",
        "ddlBloqueio": "-1",
        "ddlDia": "Qualquer dia",
        "txtHoraInicio": "",
        "txtHoraFim": "",
        "ddlTurno": "-1",
        "ddlDeptoSolicitante": "-1",
        "ddlNumResultados": "25",
        "ddlExtensao": "Texto",
        "btnDownload": "Baixar",
        "hiddenInputToUpdateATBuffer_CommonToolkitScripts": "1",
        "__EVENTTARGET": "",
        "__EVENTARGUMENT": "",
        "__LASTFOCUS": ""
    }

    MODO = PayloadModo.MICROHORARIO

    @classmethod
    def altera_modo(cls):
        cls.MODO = PayloadModo.MICROHORARIO if cls.MODO == PayloadModo.HORARIO else PayloadModo.HORARIO

    @classmethod
    def get_modo(cls):
        return cls.MODO

    @classmethod
    def intermediario(cls) -> dict:
        if cls.MODO == PayloadModo.MICROHORARIO:
            return cls._INTERMEDIARIO
        else:
            # removendo as opções que não existem no payload para o Horarios e Salas
            return cls._remove(cls._INTERMEDIARIO)

    @classmethod
    def final(cls) -> dict:
        if cls.MODO == PayloadModo.MICROHORARIO:
            return cls._FINAL
        else:
            # removendo as opções que não existem no payload para o Horarios e Salas
            return cls._remove(cls._FINAL)

    @classmethod
    def _remove(cls, payload: dict):
        ret = payload.copy()
        del ret["txtQtdCreditos"]
        del ret["ddlBloqueio"]
        del ret["ddlTurno"]
        return ret



import requests
import warnings
from bs4 import BeautifulSoup

# typing
from typing import Optional, List, Tuple
from bs4.element import Tag


URL_EMENTA = "https://www.puc-rio.br/ferramentas/ementas/ementa.aspx?cd={codigo}"


def encontra_ementa(soup: BeautifulSoup) -> Optional[str]:
    """
    Faz o parsing do html, procurando o texto da ementa.

    Retorna o texto da ementa se encontrar, ou None

    :param soup: objeto BeautifulSoup contendo a página da ementa

    :return:
    """

    tag_ementa: Tag = soup.find(id='pEmenta', recursive=True)

    return tag_ementa.text if tag_ementa is not None else None


def encontra_prerequisitos(soup: BeautifulSoup) -> List[List[str]]:
    """
    Faz o parsing do html, procurando os grupos de prerequisitos.

    Retorna uma lista de grupos de ementas, ou None

    :param soup: objeto BeautifulSoup contendo a página da ementa
    """
    ret = []

    tag_prerequisito: Tag = soup.find(id='prerequisito', recursive=True)
    if tag_prerequisito is None:
        return ret

    for tag_grupo in tag_prerequisito.find_all('span'):
        grupo = set()
        for tag_disc in tag_grupo.find_all('a'):
            disc = tag_disc.text.strip().upper()
            if disc:
                grupo.add(disc)
        if grupo:
            ret.append(list(grupo))

    return ret


def consulta_extra(codigo: str) -> Tuple[str, List[List[str]]]:
    """
    Faz uma consulta para a página da ementa, e retorna a ementa e prerequisitos.

    Há um sleep de 0.3 segundos para evitar muitas consultas em pouco tempo ao site.

    Se não encontrar ou houver algum erro, a ementa será "Disciplina sem ementa cadastrada."

    :param codigo: código da disciplina no formato XXX0000

    :return: o texto da ementa
    """

    ementa_erro = "Disciplina sem ementa cadastrada"
    prereq_erro = []

    r = requests.get(URL_EMENTA.format(codigo=codigo))
    if r.status_code != 200:
        warnings.warn(f"Consulta da ementa da disciplina {codigo} retornou codigo {r.status_code}")
        return ementa_erro, prereq_erro

    soup = BeautifulSoup(r.text, features='html.parser')

    ementa = encontra_ementa(soup)
    prereqs = encontra_prerequisitos(soup)

    return (
        ementa.strip() if ementa is not None else ementa_erro,
        prereqs
    )



"""MicroHorario Downloader

Essa ferramenta permite um download rápido de todas as disciplinas
da PUC-Rio, sem utilizar webscraping manual ou simuladores de browser
como Selenium.

A ferramenta utiliza chamadas HTTP (GET e POST) para interagir
com o microhorário e baixar todas as disciplinas
"""

__author__ = "Daniel Guimarães (github.com/Leinadium)"
__version__ = "1.6.1"
__copyright__ = "Copyright (c) 2023 Daniel Guimarães"
__license__ = "MIT"

from time import sleep
from warnings import warn

from .consultas import consulta_inicial, consulta_intermediaria, consulta_final
from .parser import converte_para_json
from .models import RawDisciplina, Disciplina, Turma, Alocacao, Departamento, Destino
from .ementa import consulta_extra
from .payloads import PayloadMicrohorario, PayloadModo

from typing import Dict, List, Optional

__all__ = [
    "Microhorario",
    "models",
    "exceptions"
]


class Microhorario:
    """Representa a tabela completa do Microhorario.
    """

    @staticmethod
    def download():
        """Faz o download do microhorario, criando o objeto

        :rtype: Microhorario
        """
        inicio = consulta_inicial()
        inter = consulta_intermediaria(inicio)
        fim = consulta_final(inter)

        dados_crus: dict = converte_para_json(fim)

        instance: Microhorario = Microhorario(
            periodo=dados_crus['periodo'],
            emissao=dados_crus['emissao'],
            atualizacao=dados_crus['atualizacao'],
            dados_crus=dados_crus,
            departamentos=inicio.get('departamentos'),
            destinos=inicio.get('destinos')
        )
        instance._modo_fallback = PayloadMicrohorario.get_modo() == PayloadModo.HORARIO

        for rd in dados_crus['disciplinas']:    # type: RawDisciplina
            instance._add_raw_disciplina(rd)

        return instance

    def __init__(self,
                 periodo: str,
                 emissao: str,
                 atualizacao: str,
                 dados_crus: dict,
                 departamentos: Optional[dict] = None,
                 destinos: Optional[dict] = None):
        """
        Cria uma instancia do Microhorario

        Ao criar uma instancia, as disciplinas não estão preenchidas. As disciplinas só podem
        ser adicionadas através do método privado `_add_raw_disciplina`, por isso instanciar
        manualmente um microhorario não é recomendado.

        :param periodo: string representando o período atual do microhorario (20XX-X)

        :param emissao: string representando a data de emissão do microhorario atual

        :param atualizacao: string represetando a data da ultima atualização do microhorario

        :param dados_crus: o dicionario contendo as `RawDisciplina`s e as informações originais.

        :param departamentos: Caso seja um dicionario, cada chave e valor serão respectivamentes
        o código e o nome do departamento, para ser pré-adicionado.

        :param destinos: Caso seja um dicionario, cada chave e valor serão respectivamentes
        o código e o nome do destino, para ser pré adicionado.
        """

        self._periodo: str = periodo
        self._emissao: str = emissao
        self._atualizacao: str = atualizacao
        self._dados_crus: dict = dados_crus
        self._disciplinas: Dict[str, Disciplina] = dict()
        self._departamentos: Dict[str, Departamento] = dict()
        self._alocacoes: Dict[str, Alocacao] = dict()
        self._modo_fallback: bool = False

        # adicionando departamentos
        if isinstance(departamentos, dict):
            self._departamentos = {
                k: Departamento(codigo=k, nome=v) for k, v in departamentos.items()
            }

        # adicionando destinos
        if isinstance(destinos, dict):
            self._destinos: Optional[dict] = destinos
        else:
            self._destinos: Optional[dict] = None

    @property
    def is_modo_fallback(self):
        """Retorna se o microhorario está em modo fallback"""
        return self._modo_fallback

    @property
    def periodo(self):
        """O perído atual do microhorario"""
        return self._periodo

    @property
    def emissao(self):
        """A data de emissão do download do microhorario"""
        return self._emissao

    @property
    def atualizacao(self):
        """A última atualização do microhorario"""
        return self._atualizacao

    @property
    def disciplinas(self) -> List[Disciplina]:
        """Lista das disciplinas baixados"""
        return list(self._disciplinas.values())

    @property
    def departamentos(self) -> List[Departamento]:
        """Lista dos departamentos encontrados."""
        return list(self._departamentos.values())

    @property
    def raw(self):
        """Dicionario dos dados baixados, sem processamento"""
        return self._dados_crus

    def _add_raw_disciplina(self, raw: RawDisciplina):
        """Adiciona uma disciplina utilizando uma `RawDisciplina`
        """
        # criando destino
        nome_destino = raw.destino      # nome padrão é o proprio código
        if self._destinos is not None and raw.destino in self._destinos:
            nome_destino = self._destinos.get(raw.destino)
        destino = Destino(codigo=raw.destino, nome=nome_destino)

        # criando alocacao
        alocacao = Alocacao(destino=destino, vagas=raw.vaga)

        # criando turma
        turma = Turma(
            professor=raw.professor,
            codigo=raw.turma,
            turno=raw.turno,
            horario_distancia=raw.horas_distancia,
            shf=raw.shf,
            horario_e_localizacao=raw.horario_local,
            alocacoes=[alocacao]
        )

        # se ja existir a disciplina, só adiciona a turma
        if raw.codigo in self._disciplinas:
            # (se a turma ja existir, a disciplina pega a alocacao)
            self._disciplinas[raw.codigo].add_turma(turma)

        # se nao existir, tem que criar a disciplina tambem
        else:
            # se existir o departamento, coleta.
            if raw.depto in self._departamentos:
                departamento = self._departamentos[raw.depto]
            # se nao existir, cria do zero
            else:
                # pegando nome do departamento antes
                departamento = Departamento(
                    codigo=raw.depto,
                    nome=raw.depto      # nao tem como saber o nome se ele já não havia sido adicionado
                )
                self._departamentos[raw.depto] = departamento

            # cria a disciplina
            disciplina = Disciplina(
                codigo=raw.codigo,
                nome=raw.nome,
                pre_req=raw.pre_req,
                creditos=raw.creditos,
                departamento=departamento
            )
            # adiciona a turma na propria disciplina
            disciplina.add_turma(turma)

            self._disciplinas[raw.codigo] = disciplina

        return

    def as_json(self) -> dict:
        """
        Transforma o objeto em um dicionário, que é um json válido.

        :return: um dicionario contendo todas as informações do json
        """

        return {
            'periodo': self.periodo,
            'emissao': self.emissao,
            'atualizacao': self.atualizacao,
            'departamentos': [
                {
                    'nome': x.nome,
                    'codigo': x.codigo
                }
                for x in self.departamentos
            ],
            'disciplinas': [
                {
                    'nome': d.nome,
                    'codigo': d.codigo,
                    'pre_req': d.pre_req,
                    'creditos': d.creditos,
                    'departamento': d.departamento.codigo,
                    'turmas': [t.as_dict() for t in d.turmas]
                }
                for d in self.disciplinas
            ]
        }

    def coletar_extra(self, verbose=True):
        """
        Coleta as ementas e pre-requisitos de todas as disciplinas cadastradas.

        Essa função irá fazer uma chamada ao site da PUC para cada ementa e prerequisito, por isso
        o tempo de execução é longo.

        :param verbose: imprime o status atual no stdout
        """

        total = len(self._disciplinas)
        for i, (cod, disc) in enumerate(self._disciplinas.items()):       # type: int, (str, Disciplina)
            if verbose:
                print(f"\r[{i}/{total}] Coletando ementa de [{cod}]", end='')

            em, pr = "Disciplina sem ementa cadastrada.", []
            try:
                em, pr = consulta_extra(cod)
            except Exception as e:
                warn(f"Erro ao coletar ementa da disciplina {cod}: {e}")

            # convertendo para disciplinas
            disc.prerequisitos = [
                # se a disciplina nao existir, nao coloca na lista
                # afinal, tornaria impossivel de ser cumprido
                [self._disciplinas.get(x) for x in grupo if x in self._disciplinas]
                for grupo in pr
            ]
            disc.ementa = em

            sleep(0.2)



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / "README.md").read_text(encoding='utf-8')

setup(
    name='microhorario-dl',
    version='1.6.1',
    description='PUC-Rio Microhorario Downloader',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Leinadium/microhorario-dl',
    author='Daniel Guimarães <github@Leinadium>',
    author_email='daniel.sch.guima@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    install_requires=[
        "beautifulsoup4>=4",
        "requests>=2"
    ],
    python_requires=">3.7",
    project_urls={
        "Bug Reports": "https://github.com/Leinadium/microhorario-dl/issues",
        "Source": "https://github.com/Leinadium/microhorario-dl/"
    }
)



import re
from dataclasses import dataclass, asdict

# typing stuff
from typing import Optional, Dict, Match, List


RE_HORARIO = re.compile(
    r'^((?P<dia>SEG|TER|QUA|QUI|SEX|SAB|DOM)\s(?P<inicio>[0-9]{2})-(?P<fim>[0-9]{2})\s)?(?P<local>[a-zA-Z0-9]+)?$'
)


@dataclass
class Horario:
    # noinspection PyUnresolvedReferences
    """Horario da turma

    :param dia: SEG, TER, QUA, QUI, SEX, SAB ou DOM
    :type: str

    :param inicio: hora de inicio da aula (0 à 24)
    :type: int

    :param fim: hora de fim da aula (0 à 24)
    :type: int
    """
    dia: str
    inicio: int
    fim: int

    def __repr__(self):
        return f'<Horario [{self.dia} {self.inicio}-{self.fim}]>'


@dataclass
class Destino:
    # noinspection PyUnresolvedReferences
    """Destino da turma

    :param codigo: código do destino da turma (ex. QQC)
    :type: str

    :param nome: nome do destino da turma (ex. Qualquer curso)
    :type: str
    """
    codigo: str
    nome: str

    def __repr__(self):
        return f'<Destino [{self.codigo}]>'


@dataclass
class Alocacao:
    # noinspection PyUnresolvedReferences
    """Alocacao de uma turma

    :param destino: Destino da turma
    :type: Destino

    :param vagas: quantidade de vagas da turma
    :type: int
    """
    destino: Destino
    vagas: int

    def __repr__(self):
        return f'<Alocacao [{self.destino.codigo} - {self.vagas} vgs]>'


@dataclass
class RawDisciplina:
    # noinspection PyUnresolvedReferences
    """Dados de uma disciplina sem nenhum tratamento

    Esses dados estão quase exatamente como no CSV disponibilizado pelo microhorario

    Cada instancia representa na verdade uma turma, mas possui todas as informações da disciplina também.

    :arg codigo: codigo da discplina
    :type: str

    :arg nome: nome da disciplina
    :type: str

    :arg professor: nome do professor da disciplina
    :type: str

    :arg creditos: quantidade de créditos da disciplina
    :type: int

    :arg turma: código da turma
    :type: str

    :arg destino: destino da turma
    :type: str

    :arg vaga: quantidade de vagas na turma
    :type: int

    :arg turno: turno da turma
    :type: str

    :arg horario_local: string contendo horario e local da turma
    :type: str

    :arg horas_distancia: quantidade de horas à distância da turma
    :type: int

    :arg shf: quantidade de horas sem horário fixo da turma
    :type: int

    :arg pre_req: se a disciplina contém algum pré requisito
    :type: bool

    :arg depto: código do departamento da disciplina
    :type: str
    """
    codigo: str
    nome: str
    professor: str
    creditos: int
    turma: str
    destino: str
    vaga: int
    turno: str
    horario_local: str
    horas_distancia: int
    shf: int
    pre_req: bool
    depto: str

    def __repr__(self):
        return f'<RawDisciplina [{self.codigo}]>'


class Turma:
    """Representa uma turma de uma disciplina no Microhorario"""

    def __init__(self,
                 professor: str,
                 codigo: str,
                 turno: str,
                 horario_distancia: int,
                 shf: int,
                 horario_e_localizacao: str,
                 alocacoes: List[Alocacao]
                 ):

        self._professor = professor
        self._codigo = codigo
        self._turno = turno
        self._horario_distancia = horario_distancia
        self._shf = shf
        self._alocacoes: dict[str, Alocacao] = {
            x.destino.codigo: x for x in alocacoes
        }

        # fazendo parsing do horario de local
        self._lista_horarios = []
        self._localizacao = None
        self._parse_horario_localizacao(horario_e_localizacao.strip())

    def __repr__(self):
        return f'<Turma [{self.codigo}]>'

    def as_dict(self):
        """Converte a turma para um dicionario"""

        horarios = [
            asdict(x) for x in self._lista_horarios if x is not None
        ]
        localizacao = self.localizacao if self.localizacao is not None else ''
        alocacoes = [{'destino': k, 'vagas': v.vagas} for k, v in self._alocacoes]

        return {
            'professor': self.professor,
            'codigo': self.codigo,
            'turno': self.turno,
            'horario_distancia': self.horario_distancia,
            'shf': self.shf,
            'horarios': horarios,
            'localizacao': localizacao,
            'alocacoes': alocacoes
        }

    def add_alocacao(self, alocacao: Alocacao):
        if alocacao.destino.codigo not in self._alocacoes:
            self._alocacoes[alocacao.destino.codigo] = alocacao

    def _parse_horario_localizacao(self, texto: str):
        """Faz o parsing da string contendo os horarios e a localização,
        transformando em uma lista de `Horario`, e guarda em `horarios`.

        Também salva a localização do último horário encontrado.
        """
        local = None
        for t in texto.split('  '):
            m: Match = RE_HORARIO.match(t.strip())
            if m is None:
                return

            local = m.group('local')
            dia = m.group('dia')
            inicio = m.group('inicio')
            fim = m.group('fim')

            horario = Horario(dia, inicio, fim) if None not in (dia, inicio, fim) else None
            self._lista_horarios.append(horario)

        # so adiciona a ultima localizacao. pode ser um problema
        # se houver mais de uma localizaco dependendo da aula
        self._localizacao = local

    @property
    def professor(self) -> str:
        """Professor(a) que leciona a disciplina para essa turma"""
        return self._professor

    @property
    def codigo(self) -> str:
        """Código da turma"""
        return self._codigo

    @property
    def turno(self) -> str:
        """Turno da turma"""
        return self._turno

    @property
    def horarios(self) -> List[Horario]:
        """Lista de horários da turma"""
        return self._lista_horarios

    @property
    def horario_distancia(self):
        """Quantidade de horas à distância"""
        return self._horario_distancia

    @property
    def shf(self):
        """Quantidade de horas sem hora fixa."""
        return self._shf

    @property
    def localizacao(self):
        """Local onde é realizada a aula da turma"""
        return self._localizacao

    @property
    def alocacoes(self):
        """Alocacoes (vagas e destino) da turma"""
        return list(self._alocacoes.values())


@dataclass
class Departamento:
    # noinspection PyUnresolvedReferences
    """Um departamento no microhorario

    :arg codigo: codigo do departamento
    :type: str

    :arg nome: nome do departamento
    :type: str
    """

    codigo: str
    nome: str


class Disciplina:
    """Representa uma disciplina no Microhorário"""

    def __init__(self,
                 codigo: str,
                 nome: str,
                 creditos: int,
                 pre_req: bool,
                 departamento: Departamento,
                 ):

        self._codigo: str = codigo
        self._nome: str = nome
        self._creditos: int = creditos
        self._pre_req: bool = pre_req
        self._ementa: Optional[str] = None
        self._prereqs: Optional[List[List["Disciplina"]]] = None
        self._departamento = departamento
        self._turmas: Dict[str, Turma] = dict()

    def __repr__(self):
        return f'<Disciplina [{self.codigo}]>'

    @property
    def codigo(self) -> str:
        """Código da disciplina"""
        return self._codigo

    @property
    def nome(self) -> str:
        """Nome da disciplina"""
        return self._nome

    @property
    def creditos(self) -> int:
        """Quantidade de créditos da discplina"""
        return self._creditos

    @property
    def pre_req(self) -> bool:
        """Disciplina contém pré-requisitos"""
        return self._pre_req

    @property
    def ementa(self) -> Optional[str]:
        """
        Ementa da disciplina. Ela precisa ser baixada usando
        `microhorario.coletar_disciplinas()`
        """
        return self._ementa

    @ementa.setter
    def ementa(self, new_ementa: str):
        if self._ementa is None:
            self._ementa = new_ementa

    @property
    def prerequisitos(self) -> List[List["Disciplina"]]:
        """Lista de grupos de pré-requisitos da disciplina"""
        return self._prereqs

    @prerequisitos.setter
    def prerequisitos(self, new_prereqs: List[List["Disciplina"]]):
        if self._prereqs is None:
            self._prereqs = new_prereqs

    @property
    def departamento(self) -> Departamento:
        """Departamento da disciplina"""
        return self._departamento

    @property
    def turmas(self) -> List[Turma]:
        """Lista de turmas da disciplina"""
        return list(self._turmas.values())

    def add_turma(self, turma: Turma):
        """Adiciona uma turma na lista da disciplinas"""
        if turma.codigo in self._turmas:
            self._turmas[turma.codigo].add_alocacao(turma.alocacoes[0])
        else:
            self._turmas[turma.codigo] = turma



import re
import warnings

# typing stuff
from typing import List, Dict

# local imports
from .models import RawDisciplina


def get_informacoes_csv(info_str: str) -> Dict[str, str]:
    """
    Faz um parsing das informações do CSV gerado.

    Procura retirar as informações da primeira linha do CSV, no formato:
        Período: 20221;Emitido em: 05/04/2022 16:24 h; Data da última atualização: 05/04/2022 13:50h;

    :param info_str: linha contendo as informações do csv

    :return: um dicionario com as chaves "periodo", "emissao" e "atualizacao"
    """

    lista: List[str] = info_str.split(';')
    res = {
        'periodo': '',
        'emissao': '',
        'atualizacao': ''
    }

    for elemento in lista:
        # separando no ":", onde [0] tem um texto, e [1] o conteudo
        try:
            texto_e_conteudo = elemento.split(':')
            texto, conteudo = texto_e_conteudo[0], ':'.join(texto_e_conteudo[1:])
            if 'Período' in texto:
                res['periodo'] = conteudo.strip()
            elif 'Emitido' in texto:
                res['emissao'] = conteudo.strip()
            elif 'atualização' in texto:
                res['atualizacao'] = conteudo.strip()

        except ValueError:
            continue

    # verificando integridade
    for k, v in res.items():
        if not v:
            warnings.warn(f"A informacao '{k}' não foi encontrada no csv baixado")

    return res


def converte_para_json(texto_csv: str) -> dict:

    ret: dict = {}
    # le as linhas do csv
    linhas = texto_csv.splitlines()
    # salva as informacoes na linha inicial do arquivo
    ret.update(
        get_informacoes_csv(linhas[0])
    )

    # regex para o codigo, que é o primeiro texto na linha
    re_disciplina = re.compile('^[A-Z]{3}[0-9]{4}')

    lista_disciplinas: List[RawDisciplina] = list()

    for linha in linhas:
        if re_disciplina.match(linha) is None:
            continue     # pula a linha que nao tem informacao

        # splitando em brancos e retirando tambem o ';' final se tiver
        linha_split = linha.strip(' \n\r;').split(';')

        if len(linha_split) == 11:
            # caso especifico quando cai no Horarios e Salas (microhorario desligado)
            # nao existe a linha de 'créditos', 'destino' e 'vaga'
            linha_split.insert(3, '-1')     # creditos
            linha_split.insert(5, '--')     # destino
            linha_split.insert(6, '--')     # vaga

        if len(linha_split) == 14:
            # removendo horas de extensao (atualização nova do microhorario)
            linha_split.pop(11)

        if len(linha_split) != 13:
            warnings.warn(f"Linha iniciada em {linha_split[0]} está inválida: contém {len(linha_split)} elementos,"
                          f"esperado: 14, 13 ou 11")
            continue     # pula a linha que a informacao esta corrompida

        codigo = linha_split[0].strip()
        nome = linha_split[1].strip()
        professor = linha_split[2].strip()
        creditos = linha_split[3].strip()
        turma = linha_split[4].strip()
        destino = linha_split[5].strip()
        vaga = linha_split[6].strip()
        turno = linha_split[7].strip()
        horario_local = linha_split[8].strip()
        horas_distancia = linha_split[9].strip()
        shf = linha_split[10].strip()
        pre_req = linha_split[11].strip()
        depto = linha_split[12].strip()

        # fazendo parsing dos valores numericos
        creditos = int(creditos) if creditos.isnumeric() else -1
        vaga = int(vaga) if vaga.isnumeric() else -1
        horas_distancia = int(horas_distancia) if horas_distancia.isnumeric() else -1
        shf = int(shf) if shf.isnumeric() else -1

        # fazendo parsing dos valores booleanos
        pre_req = pre_req == "SIM"
        lista_disciplinas.append(RawDisciplina(
            nome=nome,
            codigo=codigo,
            professor=professor,
            creditos=creditos,
            turma=turma,
            destino=destino,
            vaga=vaga,
            turno=turno,
            horario_local=horario_local,
            horas_distancia=horas_distancia,
            shf=shf,
            pre_req=pre_req,
            depto=depto
        ))

    ret['disciplinas'] = lista_disciplinas

    return ret
